@@ -7,6 +7,7 @@ import unidecode
 from .. import dependencies
 from ... import events
 from ... import models
+from ... import engine
 
 LOG = logging.getLogger()
 router = fastapi.APIRouter(
@@ -22,7 +23,15 @@ async def api_tournaments(
     op: dependencies.DbOperator,
 ) -> tuple[models.TournamentFilter, list[models.TournamentMinimal]]:
     """List all tournaments"""
-    if filter.uid or filter.country or not filter.online or filter.states:
+    if (
+        filter.uid
+        or filter.country
+        or not filter.online
+        or filter.states
+        or filter.member_uid
+        or filter.year
+        or filter.name
+    ):
         filter = filter
     else:
         filter = None
@@ -41,7 +50,12 @@ async def api_tournaments_post(
     data.judges = await op.get_members(
         list(set([j.uid for j in data.judges]) | {member.uid})
     )
+    league = await op.get_league(data.league.uid) if data.league else None
+    # Validation happens in create_tournament which uses TournamentOrchestrator
     LOG.info("Creating new tournament: %s", data)
+    # Create a temporary orchestrator just for validation
+    temp_orchestrator = engine.TournamentOrchestrator(**dataclasses.asdict(data))
+    temp_orchestrator.update_config(data, member, league)
     uid = await op.create_tournament(data)
     return dependencies.ItemUrl(
         uid=uid, url=str(request.url_for("tournament_display", uid=uid))
@@ -50,22 +64,20 @@ async def api_tournaments_post(
 
 @router.put("/{uid}", summary="Update tournament information")
 async def api_tournament_put(
-    request: fastapi.Request,
     orchestrator: dependencies.TournamentOrchestrator,
     data: typing.Annotated[models.TournamentConfig, fastapi.Body()],
     member: dependencies.PersonFromToken,
     op: dependencies.DbOperator,
-) -> dependencies.ItemUrl:
+) -> models.Tournament:
     """Update tournament information
 
     - **uid**: The tournament unique ID
     """
     data.judges = await op.get_members([judge.uid for judge in data.judges])
-    orchestrator.update_config(data, member)
-    uid = await op.update_tournament(orchestrator)
-    return dependencies.ItemUrl(
-        uid=uid, url=str(request.url_for("tournament_display", uid=uid))
-    )
+    league = await op.get_league(data.league.uid) if data.league else None
+    orchestrator.update_config(data, member, league)  # checks member can admin
+    await op.update_tournament(orchestrator)
+    return orchestrator
 
 
 @router.get("/{uid}", summary="Get tournament data")
@@ -82,38 +94,49 @@ async def api_tournament_get(
 
 @router.get("/{uid}/info", summary="Get tournament public information")
 async def api_tournament_get_info(
-    tournament: dependencies.TournamentInfo, _: dependencies.PersonFromToken
-) -> models.TournamentInfo:
+    tournament: dependencies.TournamentInfo, member: dependencies.PersonFromToken
+) -> models.TournamentInfo | models.TournamentConfig:
     """Get tournament information
 
     - **uid**: The tournament unique ID
     """
-    return tournament
+    if member.vekn or member.uid in tournament.players:
+        return tournament
+    return models.TournamentConfig(**dataclasses.asdict(tournament))
 
 
 @router.get("/{uid}/decks", summary="Get tournament decks information")
 async def api_tournament_get_decks(
-    tournament: dependencies.TournamentConfig, member: dependencies.PersonFromToken
+    op: dependencies.DbOperator,
+    uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
+    member: dependencies.PersonFromToken,
 ) -> models.TournamentDeckInfo:
-    """Get tournament decks (organizers only)
+    """Get tournament decks
 
     - **uid**: The tournament unique ID
     """
-    dependencies.check_can_admin_tournament(member, tournament)
+    tournament = await op.get_tournament(uid)
     res = models.TournamentDeckInfo(**dataclasses.asdict(tournament))
-    if tournament.multideck:
-        for round_ in tournament.rounds:
-            for table in round_.tables:
-                for seat in table.seating:
-                    if seat.deck:
-                        res.decks.append(seat.deck)
-    else:
-        for player in tournament.players.values():
-            if player.deck:
-                res.decks.append(player.deck)
+    res.decks = engine.deck_infos(tournament, member)
     return res
 
 
+@router.get("/{uid}/report", summary="Get tournament text report")
+async def api_tournament_get_report(
+    op: dependencies.DbOperator,
+    uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
+    member: dependencies.PersonFromToken,
+) -> fastapi.responses.PlainTextResponse:
+    """Get tournament report with standings and winner decklist in TWD format
+
+    - **uid**: The tournament unique ID
+    """
+    tournament = await op.get_tournament(uid)
+    dependencies.check_can_admin_tournament(member, tournament)
+    report = dependencies.tournament_report(tournament)
+    return fastapi.responses.PlainTextResponse(content=report)
+
+
 @router.get(
     "/venue-completion/{country}/{prefix}",
     summary="Get venue completion for given country and prefiw",
