@@ -1,5 +1,7 @@
 import dataclasses
+import datetime
 import fastapi
+import logging
 import orjson
 import typing
 
@@ -8,6 +10,7 @@ from ... import geo
 from ... import models
 from ... import vekn as vekn_net
 
+LOG = logging.getLogger()
 router = fastapi.APIRouter(
     prefix="/api/vekn",
     default_response_class=fastapi.responses.ORJSONResponse,
@@ -68,9 +71,14 @@ async def api_vekn_abandon(
 
 
 async def _json_l(
+    request: fastapi.Request,
     it: typing.AsyncGenerator[any, None],
 ) -> typing.AsyncGenerator[str, None]:
     async for obj in it:
+        # stop sending if the client disconnects (reloads or changes page)
+        if await request.is_disconnected():
+            LOG.warning("request disconnected")
+            return
         yield orjson.dumps(
             obj, option=orjson.OPT_NON_STR_KEYS | orjson.OPT_APPEND_NEWLINE
         )
@@ -87,21 +95,53 @@ class JSONLResponse(fastapi.responses.ORJSONResponse):
     summary="Get all members",
 )
 async def api_vekn_members(
-    member: dependencies.PersonFromToken, op: dependencies.DbOperator
+    request: fastapi.Request,
+    member: dependencies.PersonFromToken,
+    op: dependencies.DbOperator,
+    since: dependencies.IfNoneMatch,
 ) -> fastapi.Response:
     """
-    - If you're a VEKN member, you get the whole list in a streaming response,
-      as [JSON Lines](https://jsonlines.org): `Content-Type: application/jsonl`
+    - If you're a VEKN member, you either get the whole list in a streaming response,
+      as [JSON Lines](https://jsonlines.org): `Content-Type: application/x-ndjson`,
+      or an update (list of `updated` Person, list of `deleted` Person.uid)
+      as normal JSON (`Content-Type: application/json`) if you provided a `since`.
     - If you're not, you get only the public officials (Princes and NCs) as normal JSON:
       `Content-Type: application/json`
     """
     if member.vekn:
-        return fastapi.responses.StreamingResponse(
-            _json_l(op.get_members_gen()), media_type="application/jsonl"
-        )
+        LOG.debug("fetching members since: %s", since)
+        if since:
+            timestamp, people_update = await op.get_members_since(
+                since, vekn_only=models.MemberRole.ADMIN not in member.roles
+            )
+            if not people_update:
+                return fastapi.responses.Response(status_code=304)
+            return fastapi.responses.ORJSONResponse(
+                people_update,
+                headers={
+                    "ETag": timestamp.astimezone(datetime.timezone.utc).isoformat(),
+                    "Cache-Control": "no-store",
+                },
+            )
+        else:
+            timestamp, generator = await op.get_members_generator(
+                vekn_only=models.MemberRole.ADMIN not in member.roles
+            )
+            return fastapi.responses.StreamingResponse(
+                _json_l(request, generator()),
+                media_type="application/x-ndjson",
+                headers={
+                    "ETag": timestamp.astimezone(datetime.timezone.utc).isoformat(),
+                    "Cache-Control": "no-store",
+                },
+            )
     else:
         return fastapi.responses.ORJSONResponse(
-            await op.get_externally_visible_members(member)
+            await op.get_externally_visible_members(member),
+            headers={
+                "X-Data-Scope": "public",
+                "Cache-Control": "no-store",
+            },
         )
 
 
