@@ -1,6 +1,7 @@
 import aiohttp
 import asyncio
 import base64
+import dataclasses
 import datetime
 import dotenv
 import fastapi_mail
@@ -11,13 +12,14 @@ import hmac
 import importlib
 import itsdangerous.url_safe
 import jwt
+import krcg.deck
 import logging
 import os
 import pydantic.dataclasses
 import typing
 import urllib.parse
 import uuid
-
+import zoneinfo
 
 from .. import db
 from .. import engine
@@ -28,6 +30,7 @@ from .. import vekn
 # ############################################################################### Config
 dotenv.load_dotenv()
 
+VEKN_PUSH = os.getenv("VEKN_PUSH", "")
 SESSION_KEY = os.getenv("SESSION_KEY", "dev_key")
 SITE_URL_BASE = os.getenv("SITE_URL_BASE", "http://127.0.0.1:8000")
 DISCORD_CLIENT_ID = os.getenv("DISCORD_CLIENT_ID")
@@ -92,7 +95,7 @@ class DiscordAuth:
 
 
 # ############################################################################# Database
-async def get_db_op():
+async def get_db_op() -> typing.AsyncIterator[db.Operator]:
     async with db.POOL.connection() as conn:
         yield db.Operator(conn)
 
@@ -113,7 +116,7 @@ MemberUidFromSession = typing.Annotated[
 ]
 
 
-async def get_member_from_session(
+async def get_person_from_session(
     request: fastapi.Request, member_uid: MemberUidFromSession, op: DbOperator
 ) -> models.Person:
     member = await op.get_member(member_uid, cls=models.Person)
@@ -126,7 +129,24 @@ async def get_member_from_session(
 
 #: Check we're in an authenticated session and return the member data from DB
 PersonFromSession = typing.Annotated[
-    models.Person, fastapi.Depends(get_member_from_session)
+    models.Person, fastapi.Depends(get_person_from_session)
+]
+
+
+async def get_member_from_session(
+    request: fastapi.Request, member_uid: MemberUidFromSession, op: DbOperator
+) -> models.Member:
+    member = await op.get_member(member_uid, cls=models.Member)
+    # Valid user_id in session, but member not in DB
+    if not member:
+        anonymous_session(request)
+        raise LoginRequired()
+    return member
+
+
+#: Check we're in an authenticated session and return the full member data from DB
+MemberFromSession = typing.Annotated[
+    models.Member, fastapi.Depends(get_member_from_session)
 ]
 
 
@@ -150,6 +170,7 @@ async def get_tournament_config(
 async def get_tournament_info(
     op: DbOperator,
     uid: typing.Annotated[str, fastapi.Path(title="Tournament unique ID")],
+    member: PersonFromSession,
 ) -> models.Tournament:
     return await op.get_tournament(uid, cls=models.TournamentInfo)
 
@@ -169,9 +190,9 @@ async def get_tournament_orchestrator(
 Tournament = typing.Annotated[models.Tournament, fastapi.Depends(get_tournament)]
 # Check there is a member token (not public) or filter data (players names)
 TournamentInfo = typing.Annotated[
-    models.TournamentConfig, fastapi.Depends(get_tournament_info)
+    models.TournamentInfo, fastapi.Depends(get_tournament_info)
 ]
-# This is public information
+# This is public information (no player information)
 TournamentConfig = typing.Annotated[
     models.TournamentConfig, fastapi.Depends(get_tournament_config)
 ]
@@ -183,18 +204,14 @@ TournamentOrchestrator = typing.Annotated[
 
 # ################################################################################ Utils
 async def vekn_sync(tournament: models.Tournament, rounds: int, user: models.Person):
+    if not VEKN_PUSH:
+        return
     if not user.vekn:
         raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
-    try:
-        if not tournament.extra.get("vekn_id"):
-            await vekn.upload_tournament(tournament, rounds, user.vekn)
-        if tournament.state == models.TournamentState.FINISHED:
-            await vekn.upload_tournament_result(tournament)
-    except vekn.NoVEKN:
-        raise fastapi.HTTPException(
-            fastapi.status.HTTP_503_SERVICE_UNAVAILABLE,
-            detail="VEKN is not available at the moment, please try again later.",
-        )
+    if not tournament.extra.get("vekn_id"):
+        await vekn.upload_tournament(tournament, rounds, user.vekn)
+    if tournament.state == models.TournamentState.FINISHED:
+        await vekn.upload_tournament_result(tournament)
 
 
 def async_timed_cache(duration: datetime.timedelta = datetime.timedelta(minutes=5)):
@@ -219,6 +236,38 @@ def async_timed_cache(duration: datetime.timedelta = datetime.timedelta(minutes=
     return wrapper
 
 
+async def parse_if_none_match(
+    inm_raw: str | None = fastapi.Header(None, alias="If-None-Match")
+) -> datetime.datetime | None:
+    """
+    Dependency that reads the If-None-Match header, parses it,
+    and returns a tz-aware UTC datetime.
+    Returns None if the header was not provided.
+    Raises HTTP 400 if the header is present but invalid.
+    """
+    if inm_raw is None:
+        return None
+
+    try:
+        dt = datetime.datetime.fromisoformat(inm_raw)
+    except (TypeError, ValueError):
+        raise fastapi.HTTPException(
+            status_code=400, detail=f"Invalid If-None-Match header: {inm_raw!r}"
+        )
+    # parsedate_to_datetime may return a naive datetime;
+    # RFC-1123 dates are always in GMT, so attach UTC if needed:
+    if dt.tzinfo is None:
+        dt = dt.replace(tzinfo=datetime.timezone.utc)
+
+    return dt
+
+
+#: Provide easy access to the "If-None-Match" header
+IfNoneMatch = typing.Annotated[
+    datetime.datetime | None, fastapi.Depends(parse_if_none_match)
+]
+
+
 # ################################################################################## Doc
 def custom_openapi(app: fastapi.FastAPI):
     def get_openapi():
@@ -291,16 +340,15 @@ def check_organizer(member: models.Person) -> None:
         raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
 
 
+def can_admin_tournament(member: models.Person, tournament: models.TournamentConfig):
+    return engine.can_admin_tournament(member, tournament)
+
+
 def check_can_admin_tournament(
     member: models.Person, tournament: models.TournamentConfig
 ):
-    if models.MemberRole.ADMIN in member.roles:
-        return
-    if models.MemberRole.NC in member.roles and member.country == tournament.country:
-        return
-    if member.uid in [j.uid for j in tournament.judges]:
-        return
-    raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
+    if not can_admin_tournament(member, tournament):
+        raise fastapi.HTTPException(fastapi.status.HTTP_403_FORBIDDEN)
 
 
 def check_can_admin_league(member: models.Person, league: models.League):
@@ -710,3 +758,117 @@ async def get_member_from_token(
 MemberFromToken = typing.Annotated[
     models.Member, fastapi.Depends(get_member_from_token)
 ]
+
+
+# #################################################################### Tournament Report
+def winners_deck(tournament: models.Tournament) -> krcg.deck.Deck | None:
+    """Winner's decklist as KRCG deck with relevant info filled"""
+    if not tournament.winner:
+        return None
+    winner_player = tournament.players.get(tournament.winner)
+    if not winner_player:
+        return None
+
+    # Find winner's deck
+    winner_deck = None
+    if tournament.multideck and tournament.rounds:
+        # For multideck, get the finals deck
+        finals_round = tournament.rounds[-1]
+        for table in finals_round.tables:
+            for seat in table.seating:
+                if seat.player_uid == tournament.winner:
+                    winner_deck = seat.deck
+                    break
+    else:
+        winner_deck = winner_player.deck
+
+    if not winner_deck:
+        return None
+    deck = krcg.deck.Deck()
+    deck.from_json(dataclasses.asdict(winner_deck))
+    deck.event = tournament.name
+    deck.event_link = urllib.parse.urljoin(
+        SITE_URL_BASE, f"/tournaments/{tournament.uid}"
+    )
+    # Location (Online or Country)
+    if tournament.online:
+        deck.place = "Online"
+    elif tournament.country:
+        deck.place = tournament.country
+
+    # Date formatting
+    if tournament.finish:
+        finish_dt = tournament.finish.replace(
+            tzinfo=zoneinfo.ZoneInfo(tournament.timezone)
+        )
+        deck.date = finish_dt.date()
+    else:
+        start_dt = tournament.start.replace(
+            tzinfo=zoneinfo.ZoneInfo(tournament.timezone)
+        )
+        deck.date = start_dt.date()
+
+    # Format (2R+F or 3R+F)
+    num_rounds = len(tournament.rounds)
+    if num_rounds > 0:
+        if tournament.finals_seeds and num_rounds <= 4:
+            deck.tournament_format = f"{num_rounds - 1}R+F"
+        elif num_rounds <= 3:
+            deck.tournament_format = f"{num_rounds}R"
+
+    # Number of active players
+    active_players = [p for p in tournament.players.values() if p.rounds_played > 0]
+    deck.players_count = len(active_players)
+
+    # Winner name
+    if tournament.winner:
+        winner = tournament.players.get(tournament.winner)
+        deck.player = winner.name
+
+    return deck
+
+
+def tournament_report(tournament: models.Tournament) -> str:
+    """Generate a human-readable text report of the tournament."""
+    report = []
+
+    # Tournament name
+    report.append(f"Tournament report: {tournament.name}")
+    report.append("")
+
+    # Standings
+    report.append("=" * 80)
+    report.append("STANDINGS")
+    report.append("=" * 80)
+    report.append("")
+
+    standings_list = engine.standings(tournament)
+    if standings_list:
+        # Header
+        report.append(f"{'Rank':<6} {'Name':<30} {'Country':<20} {'Score':<15}")
+        report.append("-" * 80)
+
+        for rank, player in standings_list:
+            score_str = (
+                f"{player.result.gw}GW{player.result.vp}"
+                if player.result.gw
+                else f"{player.result.vp}VP"
+            )
+            country_str = player.country or ""
+            report.append(
+                f"{rank:<6} {player.name:<30} {country_str:<20} {score_str:<15}"
+            )
+
+    report.append("")
+    report.append("")
+
+    deck = winners_deck(tournament)
+    if deck:
+        report.append("=" * 80)
+        report.append("WINNER'S DECKLIST")
+        report.append("=" * 80)
+        report.append("")
+        report.append(deck.to_txt("twd"))
+        report.append("")
+
+    return "\n".join(report)
\ No newline at end of file
