@@ -1,4 +1,5 @@
 import dataclasses
+import datetime
 import fastapi
 import fastapi.encoders
 import fastapi.templating
@@ -16,6 +17,15 @@ from ... import scoring
 
 LOG = logging.getLogger()
 
+# Cache the package version for asset URLs
+try:
+    from importlib.metadata import version
+
+    _PKG_VERSION = version("vtes-archon")
+except Exception:
+    # Fallback for development
+    _PKG_VERSION = "dev"
+
 
 def jsonable(obj: typing.Any) -> typing.Any:
     """Useful filter for Jinja templates: `{{ data | jsonable | tojson }}`"""
@@ -28,6 +38,16 @@ def country_with_flag(country_name: str) -> str:
     return geo.COUNTRIES_BY_NAME[country_name].flag + " " + country_name
 
 
+def asset_url(filename: str) -> str:
+    """
+    Resolve an asset filename with version-based cache busting.
+
+    Usage in templates: {{ asset_url('login.js') }}
+    Returns: filename?v=version (e.g., "login.js?v=1.2.3")
+    """
+    return f"{filename}?v={_PKG_VERSION}"
+
+
 def __init_templates() -> fastapi.templating.Jinja2Templates:
     """Initialize Jinja2 templates engine"""
     with importlib.resources.path("archon", "templates") as templates:
@@ -36,6 +56,8 @@ def __init_templates() -> fastapi.templating.Jinja2Templates:
         )
         templates.env.filters["jsonable"] = jsonable
         templates.env.filters["country_with_flag"] = country_with_flag
+        # Add asset_url as a global function for version-based cache busting
+        templates.env.globals["asset_url"] = asset_url
         return templates
 
 
@@ -56,14 +78,77 @@ async def html_auth_oauth(
     client_id: typing.Annotated[str, fastapi.Query()],
     redirect_uri: typing.Annotated[str, fastapi.Query()],
     state: typing.Annotated[str, fastapi.Query()],
-    member_uid: dependencies.MemberUidFromSession,
+    member: dependencies.MemberFromSession,
+    op: dependencies.DbOperator,
 ):
-    # TODO: display a page asking for the user's authorization
-    next = fastapi.datastructures.URL(redirect_uri)
-    next = next.include_query_params(
-        state=state, code=dependencies.create_authorization_code(client_id, member_uid)
+    # Check if user has already authorized this client
+    if client_id in member.authorized_clients:
+        # User already authorized, redirect with code
+        next = fastapi.datastructures.URL(redirect_uri)
+        next = next.include_query_params(
+            state=state,
+            code=dependencies.create_authorization_code(
+                client_id, member.uid, redirect_uri
+            ),
+        )
+        return fastapi.responses.RedirectResponse(next)
+
+    # Get client info for consent page
+    client = await op.get_client(client_id)
+    if not client:
+        raise fastapi.HTTPException(status_code=404, detail="Client not found")
+
+    # Show consent page
+    return TEMPLATES.TemplateResponse(
+        request=fastapi.Request,
+        name="oauth/consent.html.j2",
+        context={
+            "client_id": client_id,
+            "client_name": client.name,
+            "redirect_uri": redirect_uri,
+            "state": state,
+            "member": member,
+        },
     )
-    return fastapi.responses.RedirectResponse(next)
+
+
+@router.post(
+    "/auth/oauth/consent",
+    summary="Handle user consent for OAuth authorization",
+    tags=["oauth"],
+)
+async def html_auth_oauth_consent(
+    request: fastapi.Request,
+    client_id: typing.Annotated[str, fastapi.Form()],
+    redirect_uri: typing.Annotated[str, fastapi.Form()],
+    state: typing.Annotated[str, fastapi.Form()],
+    action: typing.Annotated[str, fastapi.Form()],
+    member: dependencies.MemberFromSession,
+    op: dependencies.DbOperator,
+):
+    if action == "allow":
+        # Store user authorization
+        member.authorized_clients[client_id] = {
+            "authorized_at": datetime.datetime.now(tz=datetime.timezone.utc).isoformat()
+        }
+        await op.update_member(member)
+
+        # Redirect with authorization code
+        next = fastapi.datastructures.URL(redirect_uri)
+        next = next.include_query_params(
+            state=state,
+            code=dependencies.create_authorization_code(
+                client_id, member.uid, redirect_uri
+            ),
+        )
+        return fastapi.responses.RedirectResponse(next)
+    else:
+        # User denied access, redirect with error
+        next = fastapi.datastructures.URL(redirect_uri)
+        next = next.include_query_params(
+            state=state, error="access_denied", error_description="User denied access"
+        )
+        return fastapi.responses.RedirectResponse(next)
 
 
 @router.post(
@@ -76,14 +161,39 @@ async def html_auth_oauth_token(
     grant_type: typing.Annotated[str, fastapi.Form()],
     code: typing.Annotated[str, fastapi.Form()],
     client_uid: dependencies.ClientLogin,
+    op: dependencies.DbOperator,
 ):
     if grant_type != "authorization_code":
         raise fastapi.HTTPException(status_code=403)
     member_uid = dependencies.check_authorization_code(client_uid, code)
+
+    # Verify user has authorized this client
+    member = await op.get_member(member_uid, cls=models.Member)
+    if client_uid not in member.authorized_clients:
+        raise fastapi.HTTPException(status_code=403, detail="Client not authorized")
+
     access_token = dependencies.create_access_token(member_uid)
     return dependencies.Token(access_token=access_token, token_type="Bearer")
 
 
+@router.post(
+    "/auth/oauth/revoke",
+    summary="Revoke authorization for a client",
+    tags=["oauth"],
+)
+async def html_auth_oauth_revoke(
+    client_uid: typing.Annotated[str, fastapi.Form()],
+    member: dependencies.MemberFromSession,
+    op: dependencies.DbOperator,
+):
+    if client_uid not in member.authorized_clients:
+        raise fastapi.HTTPException(status_code=404, detail="Authorization not found")
+
+    member.authorized_clients.pop(client_uid, None)
+    await op.update_member(member)
+    return {"message": "Authorization revoked successfully"}
+
+
 @router.get(
     "/auth/discord",
     summary="Endpoint where the user is redirected after having logged in Discord",
@@ -331,14 +441,28 @@ def _filter(cls, filter_cls, obj):
 @router.get("/tournament/{uid}/display.html")
 async def tournament_display(
     request: fastapi.Request,
+    uid: typing.Annotated[str, fastapi.Path()],
     context: dependencies.SessionContext,
-    tournament: dependencies.TournamentInfo,
+    op: dependencies.DbOperator,
 ):
-    request.session["next"] = str(
-        request.url_for("tournament_display", uid=tournament.uid)
-    )
-    # filter out other members info
+    # get whole tournament information, filter depending on membership
     member_uid = request.session.get("user_id", None)
+    tournament = await op.get_tournament(uid)
+    if member_uid:
+        context["deck_infos"] = engine.deck_infos(tournament, context["member"])
+    # non-members get only public info
+    if not member_uid:
+        context["tournament"] = models.TournamentConfig(
+            **dataclasses.asdict(tournament)
+        )
+        return TEMPLATES.TemplateResponse(
+            request=request,
+            name="tournament/display.html.j2",
+            context=context,
+        )
+    # filter out private/organizer info
+    tournament = models.TournamentInfo(**dataclasses.asdict(tournament))
+    # now filter out other members info depending on standings mode
     provide_score = set()
     if member_uid:
         provide_score.add(member_uid)
@@ -355,15 +479,15 @@ async def tournament_display(
                 break
         context["cutoff"] = cutoff
     for k, v in tournament.players.items():
-        if (
+        if k == member_uid:
+            pass
+        elif (
             tournament.standings_mode == models.StandingsMode.PUBLIC
             or tournament.state
             in [models.TournamentState.FINALS, models.TournamentState.FINISHED]
             or k in provide_score
         ):
             tournament.players[k] = _filter(models.Player, models.PlayerInfo, v)
-        elif k == member_uid:
-            pass
         else:
             tournament.players[k] = _filter(models.Player, models.PublicPerson, v)
     # multideck tournaments: keep the deck info per round only for the player asking
@@ -557,6 +681,9 @@ async def member_display(
     if member.uid != uid and not member.vekn:
         target = await op.get_member(uid)
         dependencies.check_can_contact(member, target)
+
+    # No need to add authorized apps to context - handled in TypeScript
+
     return TEMPLATES.TemplateResponse(
         request=request,
         name="member/display.html.j2",
