@@ -2,11 +2,14 @@ import asyncio
 import contextlib
 import dotenv
 import fastapi
+import fastapi.exceptions
 import fastapi.responses
 import fastapi.staticfiles
 import importlib.resources
+import krcg.vtes
 import logging
 import os
+import psycopg.errors
 import starlette.middleware.sessions
 import uvicorn.logging
 
@@ -55,6 +58,7 @@ async def sync_vekn() -> int | None:
         return 1
     async with db.operator(autocommit=True) as op:
         await op.purge_tournament_events()
+        await op.close_old_tournaments()
         await sync_vekn_members(op)
         members = await op.get_members_vekn_dict()
         async for event in vekn.get_events_serial(members):
@@ -89,6 +93,7 @@ async def lifespan(app: fastapi.FastAPI):
         # sync VEKN asynchronously, start in the meantime
         task = asyncio.create_task(sync_vekn())
         task.add_done_callback(log_sync_errors)
+        krcg.vtes.VTES.load()
         yield
         task.cancel()
     LOG.debug("Exiting APP lifespan")
@@ -186,3 +191,45 @@ def db_exception_handler(
     else:
         LOG.warning(exc.args[0])
     return fastapi.responses.JSONResponse({"detail": exc.args[0]}, 400)
+
+
+# pydantic validation errors with detailed messages
+@app.exception_handler(fastapi.exceptions.RequestValidationError)
+def validation_exception_handler(
+    request: fastapi.Request, exc: fastapi.exceptions.RequestValidationError
+) -> fastapi.responses.JSONResponse:
+    """
+    Returns detailed validation error messages
+    """
+    LOG.warning("Validation error: %s", exc.errors())
+    return fastapi.responses.JSONResponse({"detail": exc.errors()}, 422)
+
+
+# DB issue
+@app.exception_handler(psycopg.errors.Error)
+def psycopg_exception_handler(
+    request: fastapi.Request, exc: psycopg.errors.Error
+) -> fastapi.responses.JSONResponse:
+    """
+    Returns a well-serialized JSON error message
+    """
+    if __debug__:
+        LOG.exception("Database error", exc_info=exc)
+    else:
+        LOG.warning(exc.args[0])
+    return fastapi.responses.JSONResponse({"detail": str(exc)}, 500)
+
+
+# Runtime errors
+@app.exception_handler(RuntimeError)
+def default_exception_handler(
+    request: fastapi.Request, exc: RuntimeError
+) -> fastapi.responses.JSONResponse:
+    """
+    Returns a well-serialized JSON error message
+    """
+    if __debug__:
+        LOG.exception("runtime error", exc_info=exc)
+    else:
+        LOG.warning(exc.args[0])
+    return fastapi.responses.JSONResponse({"detail": str(exc)}, 500)
