@@ -68,8 +68,16 @@ async def init():
                 "CREATE TABLE IF NOT EXISTS members("
                 "uid UUID DEFAULT gen_random_uuid() PRIMARY KEY, "
                 "vekn TEXT DEFAULT '', "
+                "last_updated TIMESTAMP WITH TIME ZONE "
+                "DEFAULT now(), "
                 "data jsonb)"
             )
+            # TODO; remove after migration
+            await cursor.execute(
+                "ALTER TABLE members "
+                "ADD COLUMN IF NOT EXISTS "
+                "last_updated TIMESTAMP WITH TIME ZONE DEFAULT now()"
+            )
             # Unique index on discord ID
             await cursor.execute(
                 "CREATE UNIQUE INDEX IF NOT EXISTS idx_member_discord_id "
@@ -77,16 +85,22 @@ async def init():
                 "USING BTREE ((data -> 'discord' ->> 'id'::text))"
             )
             # Unique index on email
+            # TODO remove after migration
+            await cursor.execute("DROP INDEX IF EXISTS idx_member_email")
             await cursor.execute(
                 "CREATE UNIQUE INDEX IF NOT EXISTS idx_member_email "
                 "ON members "
-                "USING BTREE ((data ->> 'email'::text))"
+                "USING BTREE ((data ->> 'email'::text)) "
+                "WHERE (data->>'email') IS NOT NULL AND (data->>'email') <> ''::text "
             )
             # Index vekn ID#
+            # TODO remove after migration
+            await cursor.execute("DROP INDEX IF EXISTS idx_member_email")
             await cursor.execute(
-                "CREATE INDEX IF NOT EXISTS idx_member_vekn "
+                "CREATE UNIQUE INDEX IF NOT EXISTS idx_member_vekn "
                 "ON members "
-                "USING BTREE (vekn)"
+                "USING BTREE (vekn) "
+                "WHERE vekn IS NOT NULL AND vekn <> ''::text "
             )
             # Indexes for fast ranking queries
             for category in models.RankingCategoy:
@@ -108,6 +122,65 @@ async def init():
                 "ON members "
                 "USING GIST ((data ->> 'name') gist_trgm_ops)"
             )
+            # Index last_updated
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_member_last_updated "
+                "ON members "
+                "USING BTREE (last_updated)"
+            )
+            # keep last_updated up to date automatically
+            await cursor.execute(
+                "CREATE OR REPLACE FUNCTION update_last_updated_if_changed() "
+                "RETURNS TRIGGER AS $$ "
+                "DECLARE "
+                "new_json jsonb; "
+                "old_json jsonb; "
+                "BEGIN "
+                "new_json := to_jsonb(NEW) - 'last_updated'; "
+                "old_json := to_jsonb(OLD) - 'last_updated'; "
+                ""
+                "IF new_json IS DISTINCT FROM old_json THEN "
+                "    NEW.last_updated = now(); "
+                "END IF; "
+                ""
+                "RETURN NEW; "
+                "END; "
+                "$$ LANGUAGE plpgsql; "
+            )
+            await cursor.execute(
+                "CREATE OR REPLACE TRIGGER set_last_updated "
+                "BEFORE UPDATE ON members "
+                "FOR EACH ROW "
+                "EXECUTE FUNCTION update_last_updated_if_changed(); "
+            )
+            # log members deletions
+            await cursor.execute(
+                "CREATE TABLE IF NOT EXISTS member_deletions( "
+                "uid UUID PRIMARY KEY, "
+                "deleted_at TIMESTAMP WITH TIME ZONE)"
+            )
+            # Index deleted_at
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_member_deletions_deleted_at "
+                "ON member_deletions "
+                "USING BTREE (deleted_at)"
+            )
+            await cursor.execute(
+                "CREATE OR REPLACE FUNCTION log_member_deletion() "
+                "RETURNS TRIGGER AS $$ "
+                "BEGIN "
+                "INSERT INTO member_deletions (uid, deleted_at) "
+                "VALUES (OLD.uid, now()); "
+                "RETURN OLD; "
+                "END; "
+                "$$ LANGUAGE plpgsql; "
+            )
+            await cursor.execute(
+                "CREATE OR REPLACE TRIGGER track_member_deletion "
+                "AFTER DELETE ON members "
+                "FOR EACH ROW "
+                "EXECUTE FUNCTION log_member_deletion()"
+            )
             # ############################################################## tournaments
             await cursor.execute(
                 "CREATE TABLE IF NOT EXISTS tournaments("
@@ -136,6 +209,12 @@ async def init():
                 "AS $$select ($1 || ' ' || $2)::timestamptz$$ "
                 "LANGUAGE SQL IMMUTABLE RETURNS NULL ON NULL INPUT"
             )
+            # year extraction function for indexing
+            await cursor.execute(
+                "CREATE OR REPLACE FUNCTION year_from_timetz(text, text) RETURNS integer "
+                "AS $$select EXTRACT(YEAR FROM ($1 || ' ' || $2)::timestamptz)::integer$$ "
+                "LANGUAGE SQL IMMUTABLE RETURNS NULL ON NULL INPUT"
+            )
             await cursor.execute(
                 "CREATE INDEX IF NOT EXISTS idx_tournament_start "
                 "ON tournaments "
@@ -144,6 +223,16 @@ async def init():
                 "(uid::text)"
                 ")"
             )
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_tournament_year "
+                "ON tournaments "
+                "USING BTREE ((year_from_timetz(data ->> 'start', data ->> 'timezone')))"
+            )
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_tournament_name_trgm "
+                "ON tournaments "
+                "USING GIST ((data ->> 'name') gist_trgm_ops)"
+            )
             await cursor.execute(
                 "CREATE INDEX IF NOT EXISTS idx_tournament_country "
                 "ON tournaments "
@@ -172,6 +261,16 @@ async def init():
                 "ON leagues "
                 "USING BTREE ((finish), (uid::text))"
             )
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_league_parent "
+                "ON leagues "
+                "USING BTREE((data->'parent'->>'uid'::text))"
+            )
+            await cursor.execute(
+                "CREATE INDEX IF NOT EXISTS idx_league_type "
+                "ON leagues "
+                "USING BTREE((data->>'kind'::text))"
+            )
             # ################################################################## clients
             await cursor.execute(
                 "CREATE TABLE IF NOT EXISTS clients("
@@ -207,12 +306,15 @@ async def reset(keep_members: bool = True):
             LOG.warning("Reset DB")
             await cursor.execute("DROP TABLE IF EXISTS tournament_events")
             await cursor.execute("DROP TABLE IF EXISTS tournaments")
+            await cursor.execute("DROP TABLE IF EXISTS leagues")
+            await cursor.execute("DROP TABLE IF EXISTS clients")
             if not keep_members:
                 await cursor.execute("DROP TABLE IF EXISTS members")
+                await cursor.execute("DROP TABLE IF EXISTS member_deletions")
 
 
 T = typing.TypeVar("T", bound=models.TournamentMinimal)
-P = typing.TypeVar("T", bound=models.PublicPerson)
+P = typing.TypeVar("P", bound=models.PublicPerson)
 
 
 @contextlib.asynccontextmanager
@@ -274,6 +376,18 @@ class Operator:
             stored_hash = (await res.fetchone())[0]
         return hmac.compare_digest(secret_hash, stored_hash)
 
+    async def get_client(self, client_uid: str) -> models.Client | None:
+        """Get a client by uid"""
+        async with self.conn.cursor() as cursor:
+            res = await cursor.execute(
+                "SELECT data FROM clients WHERE uid=%s",
+                [client_uid],
+            )
+            data = await res.fetchone()
+            if data:
+                return self._instanciate(data[0], models.Client)
+            return None
+
     async def create_tournament(self, tournament: models.TournamentConfig) -> str:
         """Create a tournament, returns its uid"""
         uid = uuid.uuid4()
@@ -299,9 +413,13 @@ class Operator:
             )
             data = await res.fetchone()
             if data:
-                # TODO remove the "players" check once the vekn sync is stabilized
+                # TODO remove the "external" check once the vekn sync is stabilized
                 # we don't want to overwrite, see update tournament
-                if not data[0]["extra"].get("external") or data[0]["players"]:
+                # we overwrite tournaments finished in vekn archon and not here
+                if not data[0]["extra"].get("external") and (
+                    not tournament.state == models.TournamentState.FINISHED
+                    or data[0]["state"] == models.TournamentState.FINISHED
+                ):
                     return
                 uid = uuid.UUID(data[0]["uid"])
                 tournament.uid = str(uid)
@@ -347,7 +465,7 @@ class Operator:
         args = []
         if filter:
             # Note: bad practice to use OFFSET, it becomes inefficient the larger it is
-            # using cursor in the WHERE clause with the appropriate index is the way to go.
+            # using cursor in the WHERE clause with the appropriate index is the way.
             Q += " WHERE "
             if filter.uid:
                 pieces.append(
@@ -367,6 +485,27 @@ class Operator:
             if filter.states:
                 pieces.append("data ->> 'state' = ANY(%s)")
                 args.append(filter.states)
+            if filter.member_uid:
+                pieces.append(
+                    "("
+                    "data->'players' ? %s OR "
+                    "data->'judges' @> %s"
+                    ")"
+                )
+                args.extend(
+                    [
+                        filter.member_uid,
+                        psycopg.types.json.Jsonb([{"uid": filter.member_uid}]),
+                    ]
+                )
+            if filter.year:
+                pieces.append(
+                    "year_from_timetz(data ->> 'start', data ->> 'timezone') = %s"
+                )
+                args.append(filter.year)
+            if filter.name and len(filter.name) > 2:
+                pieces.append("data ->> 'name' ILIKE %s")
+                args.append(f"%{filter.name}%")
         Q += " AND ".join(pieces)
         Q += (
             " ORDER BY timetz(data ->> 'start', data ->> 'timezone') DESC, uid DESC "
@@ -382,6 +521,9 @@ class Operator:
                 ret_filter.country = filter.country
                 ret_filter.online = filter.online
                 ret_filter.states = filter.states
+                ret_filter.member_uid = filter.member_uid
+                ret_filter.year = filter.year
+                ret_filter.name = filter.name
             if ret and len(ret) == 100:
                 ret_filter.date = ret[-1].start.isoformat() + " " + ret[-1].timezone
                 ret_filter.uid = ret[-1].uid
@@ -433,21 +575,31 @@ class Operator:
             # if we update it, we take ownership
             # TODO: remove once we become source of truth
             tournament.extra.pop("external", None)
+            # denormalize ratings inside the tournament object itself
+            # for any update on a finished tournament
+            tournament_ratings = {}
+            if tournament.state == models.TournamentState.FINISHED:
+                tournament_ratings = engine.ratings(tournament)
+                for player in tournament.players.values():
+                    if rating := tournament_ratings.get(player.uid):
+                        player.rating_points = rating.rating_points
+                    else:
+                        player.rating_points = None
             res = await cursor.execute(
                 "UPDATE tournaments SET data=%s WHERE uid=%s",
                 [self._jsonize(tournament), uid],
             )
             if res.rowcount < 1:
                 raise KeyError(f"Tournament {uid} not found")
-            if tournament.state == models.TournamentState.FINISHED:
+            if tournament_ratings:
                 await cursor.executemany(
-                    f"""UPDATE members 
-                    SET data=jsonb_set(data, '{{ratings,{tournament.uid}}}', %s, true) 
+                    f"""UPDATE members
+                    SET data=jsonb_set(data, '{{ratings,{tournament.uid}}}', %s, true)
                     WHERE uid=%s
                     """,
                     [
                         [jsonize(rating), uuid.UUID(uid)]
-                        for uid, rating in engine.ratings(tournament).items()
+                        for uid, rating in tournament_ratings.items()
                     ],
                 )
             return str(uid)
@@ -455,12 +607,16 @@ class Operator:
     async def delete_tournament(self, uid: str) -> None:
         """Delete a tournament"""
         async with self.conn.cursor() as cursor:
+            # take a lock on the tournament
+            await cursor.execute(
+                "SELECT uid FROM tournaments WHERE uid=%s FOR UPDATE", [uuid.UUID(uid)]
+            )
             res = await cursor.execute(
                 "SELECT jsonb_object_keys(data->'players') "
-                "FROM tournaments WHERE uid=%s FOR UPDATE",
+                "FROM tournaments WHERE uid=%s",  # no FOR UPDATE allowed because of aggregate
                 [uuid.UUID(uid)],
             )
-            player_uids = res.fetchall()
+            player_uids = await res.fetchall()
             res = await cursor.execute(
                 "DELETE FROM tournaments WHERE uid=%s", [uuid.UUID(uid)]
             )
@@ -498,6 +654,26 @@ class Operator:
             )
             return res.rowcount
 
+    async def close_old_tournaments(self) -> int:
+        """Close tournaments that are > 30 days old and not finished.
+        Returns the number of tournaments closed.
+        """
+        cutoff = datetime.datetime.now(datetime.timezone.utc)
+        cutoff -= datetime.timedelta(days=30)
+        async with self.conn.cursor() as cursor:
+            res = await cursor.execute(
+                "UPDATE tournaments "
+                "SET data = jsonb_set(data, '{state}', %s) "
+                "WHERE data->>'state' != %s "
+                "AND timetz(data ->> 'start', data ->> 'timezone') < %s",
+                [
+                    psycopg.types.json.Jsonb(models.TournamentState.FINISHED),
+                    models.TournamentState.FINISHED,
+                    cutoff,
+                ],
+            )
+            return res.rowcount
+
     async def insert_members(self, members: list[models.Member]) -> None:
         """Insert members from VEKN.
         Modify the members list in place to match DB (ie. uuids).
@@ -505,31 +681,12 @@ class Operator:
         async with self.conn.cursor() as cursor:
             # get existing VEKN, lock the table
             res = await cursor.execute(
-                "SELECT vekn, data FROM members WHERE vekn=ANY(%s) FOR UPDATE",
+                "SELECT vekn FROM members WHERE vekn=ANY(%s)",
                 [[m.vekn for m in members]],
             )
-            existing = await res.fetchall()
-            existing = {e[0]: self._instanciate(e[1], models.Member) for e in existing}
-            # do not overwrite local data & lose relevant info (sanctions, login, etc.)
-            # don't revert changes on name
-            # TODO: also don't revert country, city, roles and sponsor changes
-            # once we're stable on this
-            for i, m in enumerate(members):
-                if m.vekn in existing:
-                    local: models.Member = existing[m.vekn]
-                    # TODO remove country & city
-                    local.country = m.country
-                    local.city = m.city
-                    local.roles = m.roles
-                    local.prefix = m.prefix
-                    local.sponsor = None
-                    members[i] = local
-            # update existing
-            # cannot run two prepared statements in parallel, just wait
-            await cursor.executemany(
-                "UPDATE members SET data=%s WHERE vekn=%s",
-                [[self._jsonize(m), m.vekn] for m in members if m.vekn in existing],
-            )
+            existing = {e[0] for e in await res.fetchall()}
+            # do not overwrite existing members
+            members = [m for m in members if m.vekn not in existing]
             # insert new
             await cursor.executemany(
                 "INSERT INTO members (uid, vekn, data) VALUES (%s, %s, %s)",
@@ -579,17 +736,62 @@ class Operator:
                 )
             ]
 
-    async def get_members_gen(
-        self,
-    ) -> typing.AsyncGenerator[models.Person, None]:
+    async def get_members_since(
+        self, timestamp: datetime.datetime, vekn_only: bool
+    ) -> tuple[datetime.datetime, models.PersonsUpdate]:
         async with self.conn.cursor() as cursor:
-            async for row in cursor.stream("SELECT data FROM members"):
-                yield self._instanciate(row[0], models.Person)
+            Q = "SELECT last_updated, data FROM members WHERE last_updated > %s"
+            if vekn_only:
+                Q += " AND vekn IS NOT NULL AND vekn <> ''"
+            updated = await cursor.execute(
+                Q,
+                [timestamp],
+            )
+            updated = await updated.fetchall()
+            deleted = await cursor.execute(
+                "SELECT deleted_at, uid FROM member_deletions WHERE deleted_at > %s",
+                [timestamp],
+            )
+            deleted = await deleted.fetchall()
+            if deleted or updated:
+                last_modified = max(
+                    datetime.datetime.min.replace(tzinfo=datetime.timezone.utc),
+                    *(row[0] for row in updated),
+                    *(row[0] for row in deleted),
+                )
+                LOG.info("last modified from DB: %s", last_modified)
+            else:
+                return None, None
+            return last_modified, models.PersonsUpdate(
+                update=[self._instanciate(row[1], models.Person) for row in updated],
+                delete=[str(row[1]) for row in deleted],
+            )
+
+    async def get_members_generator(self, vekn_only: bool) -> tuple[
+        datetime.datetime,
+        typing.Callable[[None], typing.AsyncGenerator[models.Person, None]],
+    ]:
+        """Streams all members. returns a 'fuzzy' timestamp in the past."""
+        async with self.conn.cursor() as cursor:
+            timestamp_data = await cursor.execute(
+                "SELECT (now() - INTERVAL '1 hour')::timestamptz"
+            )
+            timestamp = (await timestamp_data.fetchone())[0]
+        Q = "SELECT data FROM members"
+        if vekn_only:
+            Q += " WHERE vekn IS NOT NULL AND vekn <> ''"
+
+        async def generator():
+            async with self.conn.cursor() as cursor:
+                async for row in cursor.stream(Q):
+                    yield self._instanciate(row[0], models.Person)
+
+        return timestamp, generator
 
     async def get_ranked_members(
         self, category: models.RankingCategoy
     ) -> list[models.Person]:
-        """Get members with a prominant rank in any category"""
+        """Get members with a prominant rank in a category"""
         async with self.conn.cursor() as cursor:
             return [
                 self._instanciate(data[0], models.Person)
@@ -613,11 +815,11 @@ class Operator:
 
     async def get_externally_visible_members(
         self, user: models.Person
-    ) -> list[models.Person]:
+    ) -> list[models.PublicPerson]:
         """Get all members that can be contacted by non-vekn members"""
         async with self.conn.cursor() as cursor:
             return [
-                self._instanciate(data[0], models.Person)
+                self._instanciate(data[0], models.PublicPerson)
                 async for data in cursor.stream(
                     "SELECT data FROM members "
                     "WHERE data->'roles' ? 'NC' OR data->'roles' ? 'Prince' OR uid=%s",
@@ -953,32 +1155,7 @@ class Operator:
                 tournament = self._instanciate(row[0], models.Tournament)
                 for uid, rating in engine.ratings(tournament).items():
                     all_ratings[uid][tournament.uid] = rating
-            # clean reset for all members
-            await cursor.execute(
-                "UPDATE members SET data=jsonb_set(data, '{ratings}', '{}', true)"
-            )
-            # set all tournaments ratings
-            await cursor.executemany(
-                """UPDATE members
-                SET data=jsonb_set(data, '{ratings}', %s, false)
-                WHERE uid=%s
-                """,
-                [
-                    [
-                        psycopg.types.json.Jsonb(
-                            {k: dataclasses.asdict(v) for k, v in ratings.items()}
-                        ),
-                        uuid.UUID(uid),
-                    ]
-                    for uid, ratings in all_ratings.items()
-                ],
-            )
-            # now compute the current rankings, using results from the past 18 months
-            # only the top 8 in each category count
-            rankings_lists: dict[str, dict[models.RankingCategoy, list[int]]] = (
-                collections.defaultdict(lambda: collections.defaultdict(list))
-            )
-            # compute cutoff: 18 months before today (UTC) at 00:00
+            # compute ranking cutoff: 18 months before today (UTC) at 00:00
             cutoff = datetime.datetime.now(datetime.timezone.utc)
             cutoff = cutoff.replace(hour=0, minute=0, second=0, microsecond=0)
             cutoff = cutoff.replace(year=cutoff.year - 1)
@@ -987,55 +1164,71 @@ class Operator:
             else:
                 cutoff = cutoff.replace(year=cutoff.year - 1)
                 cutoff = cutoff.replace(month=cutoff.month + 6)
-            for uid, ratings in all_ratings.items():
-                for rating in ratings.values():
-                    if (
-                        rating.tournament.start.replace(
-                            tzinfo=zoneinfo.ZoneInfo(tournament.timezone)
-                        )
-                        < cutoff
-                    ):
-                        continue
-                    if rating.tournament.format == models.TournamentFormat.Standard:
-                        if rating.tournament.online:
-                            category = models.RankingCategoy.CONSTRUCTED_ONLINE
-                        else:
-                            category = models.RankingCategoy.CONSTRUCTED_ONSITE
-                    else:
-                        if rating.tournament.online:
-                            category = models.RankingCategoy.LIMITED_ONLINE
-                        else:
-                            category = models.RankingCategoy.LIMITED_ONSITE
-                    rankings_lists[uid][category].append(rating.rating_points)
-            del all_ratings
-            # take the top 8
-            rankings = {
-                uid: {
-                    category: sum(sorted(r, reverse=True)[:8])
-                    for category, r in res.items()
-                    if r
-                }
-                for uid, res in rankings_lists.items()
-                if res
-            }
-            del rankings_lists
-            # clean everyone's ratings, then update with the newly computed values
+            # build a temporary staging table for ratings and rankings
             await cursor.execute(
-                "UPDATE members SET data=jsonb_set(data, '{ranking}', '{}', true)"
+                "CREATE TEMP TABLE staging_ratings ( "
+                "uid UUID PRIMARY KEY, "
+                "data JSONB "
+                ")"
             )
-            await cursor.executemany(
-                """UPDATE members
-                SET data=jsonb_set(data, '{ranking}', %s, false)
-                WHERE uid=%s
-                """,
-                [
-                    [
-                        psycopg.types.json.Jsonb(ranking),
-                        uuid.UUID(uid),
-                    ]
-                    for uid, ranking in rankings.items()
-                ],
+            async with cursor.copy(
+                "COPY staging_ratings (uid, data) FROM STDIN"
+            ) as copy:
+                while all_ratings:
+                    uid, ratings = all_ratings.popitem()
+                    # compute ranking from ratings
+                    ranking = collections.defaultdict(list)
+                    for rating in ratings.values():
+                        # ignore tournaments before cutoff
+                        if (
+                            rating.tournament.start.replace(
+                                tzinfo=zoneinfo.ZoneInfo(tournament.timezone)
+                            )
+                            < cutoff
+                        ):
+                            continue
+                        if rating.tournament.format in [
+                            models.TournamentFormat.Standard,
+                            models.TournamentFormat.V5,
+                        ]:
+                            if rating.tournament.online:
+                                category = models.RankingCategoy.CONSTRUCTED_ONLINE
+                            else:
+                                category = models.RankingCategoy.CONSTRUCTED_ONSITE
+                        else:
+                            if rating.tournament.online:
+                                category = models.RankingCategoy.LIMITED_ONLINE
+                            else:
+                                category = models.RankingCategoy.LIMITED_ONSITE
+                        ranking[category].append(rating.rating_points)
+                    data = psycopg.types.json.Jsonb(
+                        {
+                            "ratings": {
+                                k: dataclasses.asdict(v) for k, v in ratings.items()
+                            },
+                            "ranking": {
+                                # take top 8
+                                category: sum(sorted(r, reverse=True)[:8])
+                                for category, r in ranking.items()
+                                if r
+                            },
+                        }
+                    )
+                    await copy.write_row([uid, data])
+            # use staging to update members, then drop staging
+            # note we need a clean update to avoid triggering last_updated unnecessarily
+            await cursor.execute(
+                "UPDATE members m "
+                "SET data = m.data || s.data "
+                "FROM staging_ratings s "
+                "WHERE m.uid = s.uid "
             )
+            await cursor.execute(
+                "UPDATE members m "
+                """SET data = data || '{"ratings": {}, "ranking": {}}'::jsonb """
+                "WHERE m.uid NOT IN (SELECT uid FROM staging_ratings)"
+            )
+            await cursor.execute("DROP TABLE staging_ratings")
 
     def _jsonize(self, obj: any):
         data = dataclasses.asdict(obj)
@@ -1057,7 +1250,7 @@ class Operator:
                 data["city_geoname_id"] = geocity.geoname_id
             else:
                 data.pop("city", None)
-        return psycopg.types.json.Json(data)
+        return psycopg.types.json.Jsonb(data)
 
     def _instanciate(self, data: dict, cls: typing.Type[T]) -> T:
         if "country_iso" in data and data["country_iso"] in geo.COUNTRIES_BY_ISO:
@@ -1074,7 +1267,7 @@ class Operator:
         return cls(**data)
 
     async def create_league(self, league: models.League) -> str:
-        """Create a league, returns its uid"""
+        """Create a league, returns its uid (validation done at API layer)"""
         uid = uuid.uuid4()
         league.uid = str(uid)
         async with self.conn.cursor() as cursor:
@@ -1099,7 +1292,10 @@ class Operator:
     async def get_league_with_tournaments(
         self, uid: str
     ) -> models.LeagueWithTournaments:
-        """Get a league, with tournaments and ranking data (no lock)"""
+        """Get a league, with tournaments, sub-leagues and ranking data
+
+        For META leagues, this includes tournaments from child leagues.
+        """
         async with self.conn.cursor() as cursor:
             res = await cursor.execute(
                 "SELECT data FROM leagues WHERE uid=%s", [uuid.UUID(uid)]
@@ -1110,12 +1306,26 @@ class Operator:
             league: models.LeagueWithTournaments = self._instanciate(
                 data[0], models.LeagueWithTournaments
             )
-            # get tournaments, latest first
+            # get child leagues if this is a meta-league
+            res = await cursor.execute(
+                "SELECT data FROM leagues "
+                "WHERE (data->'parent'->>'uid')::text=%s "
+                "ORDER BY start DESC",
+                [uid],
+            )
+            child_leagues = await res.fetchall()
+            child_league_uids = [uid]  # include parent league itself
+            for row in child_leagues:
+                child_league = self._instanciate(row[0], models.LeagueMinimal)
+                league.leagues.append(child_league)
+                child_league_uids.append(child_league.uid)
+
+            # get tournaments from this league and all child leagues, latest first
             res = await cursor.execute(
                 "SELECT data FROM tournaments "
-                "WHERE (data->'league'->>'uid')::text=%s "
+                "WHERE (data->'league'->>'uid')::text = ANY(%s) "
                 "ORDER BY timetz(data ->> 'start', data ->> 'timezone') DESC",
-                [uid],
+                [child_league_uids],
             )
             tournaments = await res.fetchall()
             for row in tournaments:
@@ -1128,6 +1338,10 @@ class Operator:
                 if tournament.state != models.TournamentState.FINISHED:
                     continue
                 ratings = engine.ratings(tournament)
+                finals_score = {
+                    seat.player_uid: seat.result
+                    for seat in tournament.rounds[-1].tables[0].seating
+                }
                 for player in tournament.players.values():
                     if player.uid not in players:
                         players[player.uid] = models.LeaguePlayer(
@@ -1140,17 +1354,27 @@ class Operator:
                         )
                     players[player.uid].tournaments.append(tournament.uid)
                     players[player.uid].score += player.result
-                    if league.ranking == models.LeagueRanking.RTP:
-                        players[player.uid].points += ratings[player.uid].rating_points
-                        key = "points"
-                    elif league.ranking == models.LeagueRanking.GP:
-                        players[player.uid].points += ratings[player.uid].gp_points
-                        key = "points"
-
-                    elif league.ranking == models.LeagueRanking.Score:
-                        players[player.uid].points = 0
-                        key = "score"
+                    # leagues subtelty: do not count finals score
+                    # if we're opting for score, it's to avoid favoring finalists
+                    # and instead encourage participation more than with RTPs
+                    if player.uid in finals_score:
+                        players[player.uid].score -= finals_score[player.uid]
+                    players[player.uid].points = 0
+                    # some players are not in ratings (registerd but did not play)
+                    if player.uid in ratings:
+                        match(league.ranking):
+                            case models.LeagueRanking.RTP:
+                                players[player.uid].points += ratings[player.uid].rating_points
+                            case models.LeagueRanking.GP:
+                                players[player.uid].points += ratings[player.uid].gp_points
 
+            match(league.ranking):
+                case models.LeagueRanking.RTP:
+                    key = "points"
+                case models.LeagueRanking.GP:
+                    key = "points"
+                case models.LeagueRanking.Score:
+                    key = "score"
             def sort(p):
                 return getattr(p, key)
 
@@ -1166,7 +1390,7 @@ class Operator:
             return league
 
     async def update_league(self, league: models.League) -> str:
-        """Update a league, returns its uid"""
+        """Update a league, returns its uid (validation done at API layer)"""
         uid = uuid.UUID(league.uid)
         async with self.conn.cursor() as cursor:
             res = await cursor.execute(
@@ -1193,13 +1417,13 @@ class Operator:
 
     async def get_leagues(
         self, filter: models.LeagueFilter | None
-    ) -> list[models.League]:
+    ) -> list[models.LeagueMinimal]:
         """Get paginated leagues, filtered and ordered by start date, 100 per page"""
         Q = "SELECT data FROM leagues "
         pieces, args = [], []
         if filter:
             # Note: bad practice to use OFFSET, it becomes inefficient the larger it is
-            # using cursor in the WHERE clause with the appropriate index is the way to go.
+            # using cursor in the WHERE clause with the appropriate index is the way.
             Q += "WHERE "
             if filter.uid:
                 pieces.append(
@@ -1223,7 +1447,7 @@ class Operator:
         )
         async with self.conn.cursor() as cursor:
             ret = [
-                self._instanciate(row[0], models.League)
+                self._instanciate(row[0], models.LeagueMinimal)
                 async for row in cursor.stream(Q, args)
             ]
             ret_filter = models.LeagueFilter()
@@ -1235,13 +1459,46 @@ class Operator:
                 ret_filter.uid = ret[-1].uid
             return (ret_filter, ret)
 
+    async def get_minimal_leagues(
+        self, kind: models.LeagueKind | None = None
+    ) -> list[models.LeagueMinimal]:
+        """Get minimal leagues, optionally filtered by type, ordered by start date"""
+        if kind:
+            # Filter by specific league type
+            Q = (
+                "SELECT data FROM leagues "
+                "WHERE (data->>'kind')::text = %s "
+                "ORDER BY timetz(data ->> 'start', data ->> 'timezone') DESC, uid DESC"
+            )
+            async with self.conn.cursor() as cursor:
+                return [
+                    self._instanciate(row[0], models.LeagueMinimal)
+                    async for row in cursor.stream(Q, [kind.value])
+                ]
+        else:
+            # Return all leagues
+            Q = (
+                "SELECT data FROM leagues "
+                "ORDER BY timetz(data ->> 'start', data ->> 'timezone') DESC, uid DESC"
+            )
+            async with self.conn.cursor() as cursor:
+                return [
+                    self._instanciate(row[0], models.LeagueMinimal)
+                    async for row in cursor.stream(Q)
+                ]
+
 
 @contextlib.asynccontextmanager
 async def operator(autocommit: bool = False) -> typing.AsyncIterator[Operator]:
-    """Yields an async DB Operator to execute DB operations in a single transaction"""
+    """Yields an async DB Operator to execute DB operations.
+
+    Does not use a transaction if autocommit=True
+    """
     async with POOL.connection() as conn:
-        if autocommit:
-            await conn.set_autocommit(True)
-        yield Operator(conn)
-        if autocommit:
-            await conn.set_autocommit(False)
+        try:
+            if autocommit:
+                await conn.set_autocommit(True)
+            yield Operator(conn)
+        finally:
+            if autocommit:
+                await conn.set_autocommit(False)
