@@ -1,5 +1,6 @@
 import collections
 import dataclasses
+import datetime
 import io
 import itertools
 import krcg.seating
@@ -7,6 +8,7 @@ import krcg.deck
 import logging
 import math
 import random
+import zoneinfo
 
 from . import geo
 from . import models
@@ -16,6 +18,16 @@ from . import scoring
 LOG = logging.getLogger()
 
 
+def can_admin_tournament(member: models.Person, tournament: models.Tournament) -> bool:
+    if models.MemberRole.ADMIN in member.roles:
+        return True
+    if models.MemberRole.NC in member.roles and tournament.country == member.country:
+        return True
+    if member.uid in [j.uid for j in tournament.judges]:
+        return True
+    return False
+
+
 class TournamentManager(models.Tournament):
     """Implements re-entring idempotent tournament logic. No validity check.
     This is the logic one needs to implement on the Client/UI side to allow for
@@ -27,8 +39,14 @@ class TournamentManager(models.Tournament):
         match ev.type:
             case events.EventType.REGISTER:
                 self.register(ev, member)
+            case events.EventType.OPEN_REGISTRATION:
+                self.open_registration(ev, member)
+            case events.EventType.CLOSE_REGISTRATION:
+                self.close_registration(ev, member)
             case events.EventType.OPEN_CHECKIN:
                 self.open_checkin(ev, member)
+            case events.EventType.CANCEL_CHECKIN:
+                self.cancel_checkin(ev, member)
             case events.EventType.CHECK_IN:
                 self.check_in(ev, member)
             case events.EventType.CHECK_EVERYONE_IN:
@@ -137,9 +155,25 @@ class TournamentManager(models.Tournament):
         if self.decklist_required:
             self.players[ev.player_uid].barriers.append(models.Barrier.MISSING_DECK)
 
+    def open_registration(
+        self, ev: events.OpenRegistration, member: models.Person
+    ) -> None:
+        self.state = models.TournamentState.REGISTRATION
+
+    def close_registration(
+        self, ev: events.CloseRegistration, member: models.Person
+    ) -> None:
+        self.state = models.TournamentState.PLANNED
+
     def open_checkin(self, ev: events.OpenCheckin, member: models.Person) -> None:
         self.state = models.TournamentState.WAITING
 
+    def cancel_checkin(self, ev: events.CancelCheckin, member: models.Person) -> None:
+        for player in self.players.values():
+            if player.state != models.PlayerState.FINISHED:
+                player.state = models.PlayerState.REGISTERED
+        self.state = models.TournamentState.REGISTRATION
+
     def check_in(self, ev: events.CheckIn, member: models.Person) -> None:
         self.players[ev.player_uid].state = models.PlayerState.CHECKED_IN
 
@@ -180,7 +214,12 @@ class TournamentManager(models.Tournament):
                 player.table = 0
                 player.seat = 0
                 if player.state != models.PlayerState.FINISHED:
-                    player.state = models.PlayerState.REGISTERED
+                    # If this is the first round and player is not checked in, drop them
+                    # This avoids mistakes when using check_everyone_in in subsequent rounds
+                    if len(self.rounds) == 1 and player.state != models.PlayerState.CHECKED_IN:
+                        player.state = models.PlayerState.FINISHED
+                    else:
+                        player.state = models.PlayerState.REGISTERED
 
     def round_alter(self, ev: events.RoundAlter, member: models.Person) -> None:
         self._alter_round(ev.round, ev.seating)
@@ -331,8 +370,13 @@ class TournamentManager(models.Tournament):
             for table in round_.tables:
                 for seat in table.seating:
                     if ev.player_uid == seat.player_uid:
+                        deck_data = deck.to_json()
+                        if not deck_data.get("author", "") and ev.attribution:
+                            deck.author = member.name
+                            deck_data["author"] = member.name
                         seat.deck = models.KrcgDeck(
-                            **deck.to_json(), vdb_link=deck.to_vdb()
+                            **deck_data,
+                            vdb_link=deck.to_vdb(),
                         )
                         break
                 else:
@@ -341,7 +385,11 @@ class TournamentManager(models.Tournament):
             else:
                 raise ValueError(f"player {ev.player_uid} not in round {ev.round}")
         else:
-            player.deck = models.KrcgDeck(**deck.to_json(), vdb_link=deck.to_vdb())
+            deck_data = deck.to_json()
+            if not deck_data.get("author", "") and ev.attribution:
+                deck_data["author"]= member.name
+                deck.author = member.name
+            player.deck = models.KrcgDeck(**deck_data, vdb_link=deck.to_vdb())
         if self.decklist_required:
             try:
                 player.barriers.remove(models.Barrier.MISSING_DECK)
@@ -603,6 +651,16 @@ class CheckinClosed(TournamentError):
         return "Check-in is closed"
 
 
+class RegistrationClosed(TournamentError):
+    def __str__(self):
+        return "Registration is closed"
+
+
+class CheckinInProgress(TournamentError):
+    def __str__(self):
+        return "Check-in is in progress"
+
+
 class ResultRecorded(TournamentError):
     def __str__(self):
         return "Impossible: results have been recorded for this round"
@@ -661,18 +719,30 @@ class BannedCards(DeckIssue):
         return f"Banned cards: {','.join(self.args[1])}."
 
 
+class LeagueCompatibilityError(TournamentError):
+    def __init__(self, message: str):
+        super().__init__(message)
+
+    def __str__(self):
+        return self.args[0]
+
+
 class TournamentOrchestrator(TournamentManager):
     """Implements all input checks and raise meaningful errors"""
 
     def update_config(
-        self, config: models.TournamentConfig, member: models.Person
+        self,
+        config: models.TournamentConfig,
+        member: models.Person,
+        league: models.League | None = None,
     ) -> None:
         self._check_judge(None, member)
-        LOG.warning("Updating tournament config: %s", config)
+        LOG.info("Updating tournament config: %s", config)
         if not config.judges:
             raise ConfigError("A tournament must have at least one judge")
         if (
-            config.format != models.TournamentFormat.Standard
+            config.format
+            not in [models.TournamentFormat.Standard, models.TournamentFormat.V5]
             and config.rank != models.TournamentRank.BASIC
         ):
             raise ConfigError("Non-Standard tournaments must have Basic rank")
@@ -682,6 +752,9 @@ class TournamentOrchestrator(TournamentManager):
             raise ConfigError("Only Basic rank tournaments can allow multideck")
         if config.multideck and config.decklist_required:
             raise ConfigError("Multideck tournaments cannot require decklists")
+        if config.league and league:
+            assert league.uid == config.league.uid
+            self._check_league_compatibility(config, member, league)
         for field in dataclasses.fields(config):
             if field.name in ["uid", "state"]:
                 continue
@@ -717,6 +790,10 @@ class TournamentOrchestrator(TournamentManager):
         if not ev.vekn:
             # only a judge can register a player without VEKN
             self._check_judge(ev, member)
+        # Non-judges can only register when state is REGISTRATION
+        if not can_admin_tournament(member, self):
+            if self.state != models.TournamentState.REGISTRATION:
+                raise RegistrationClosed(ev)
         super().register(ev, member)
 
     def _check_not_playing(self, ev: events.TournamentEvent):
@@ -728,19 +805,96 @@ class TournamentOrchestrator(TournamentManager):
             raise TournamentFinished(ev)
 
     def _check_judge(self, ev: events.TournamentEvent, member: models.Person) -> None:
-        if models.MemberRole.ADMIN in member.roles:
-            return
-        if models.MemberRole.NC in member.roles and self.country == member.country:
-            return
-        if member.uid in [j.uid for j in self.judges]:
+        if can_admin_tournament(member, self):
             return
         raise NotJudge(ev)
 
+    def _check_league_compatibility(
+        self,
+        config: models.TournamentConfig,
+        member: models.Person,
+        league: models.League,
+    ) -> None:
+        """Validate that tournament configuration is compatible with its league.
+
+        Checks format compatibility, online/onsite match, organizer permissions,
+        and date overlap (with 1-day tolerance).
+
+        Raises LeagueCompatibilityError if validation fails.
+        """
+        if member.uid not in [j.uid for j in league.organizers]:
+            self._check_judge(None, member)
+        if league.format == models.TournamentFormat.Standard and config.format not in [
+            models.TournamentFormat.Standard,
+            models.TournamentFormat.V5,
+        ]:
+            raise LeagueCompatibilityError(
+                "Standard league only accepts Standard or V5 tournaments"
+            )
+        if (
+            league.format != models.TournamentFormat.Standard
+            and config.format != league.format
+        ):
+            raise LeagueCompatibilityError(
+                "League format doesn't match tournament format"
+            )
+        if config.online and not league.online:
+            raise LeagueCompatibilityError(
+                "Online tournament cannot be part of onsite league"
+            )
+
+        # Check date overlap (with 1 day tolerance)
+        tournament_start = config.start.replace(
+            tzinfo=zoneinfo.ZoneInfo(config.timezone)
+        )
+        league_start = league.start.replace(tzinfo=zoneinfo.ZoneInfo(league.timezone))
+        tournament_finish = (config.finish or league_start).replace(
+            tzinfo=zoneinfo.ZoneInfo(config.timezone)
+        )
+        league_finish = (league.finish or tournament_start).replace(
+            tzinfo=zoneinfo.ZoneInfo(league.timezone)
+        )
+        one_day = datetime.timedelta(days=1)
+        if not (
+            tournament_start <= league_finish + one_day
+            and league_start <= tournament_finish + one_day
+        ):
+            raise LeagueCompatibilityError(
+                "Tournament dates don't overlap with league dates"
+            )
+
+    def open_registration(
+        self, ev: events.OpenRegistration, member: models.Person
+    ) -> None:
+        self._check_judge(ev, member)
+        self._check_not_playing(ev)
+        if self.state == models.TournamentState.WAITING:
+            raise CheckinInProgress(ev)
+        super().open_registration(ev, member)
+
+    def close_registration(
+        self, ev: events.CloseRegistration, member: models.Person
+    ) -> None:
+        self._check_judge(ev, member)
+        if self.state not in [
+            models.TournamentState.PLANNED,
+            models.TournamentState.REGISTRATION,
+        ]:
+            raise RegistrationClosed(ev)
+        super().close_registration(ev, member)
+
     def open_checkin(self, ev: events.OpenCheckin, member: models.Person) -> None:
         self._check_not_playing(ev)
         self._check_judge(ev, member)
         super().open_checkin(ev, member)
 
+    def cancel_checkin(self, ev: events.CancelCheckin, member: models.Person) -> None:
+        self._check_not_playing(ev)
+        if self.state == models.TournamentState.PLANNED:
+            raise CheckinClosed(ev)
+        self._check_judge(ev, member)
+        super().cancel_checkin(ev, member)
+
     def check_in(self, ev: events.CheckIn, member: models.Person) -> None:
         self._check_not_playing(ev)
         if ev.code:
@@ -992,10 +1146,14 @@ class TournamentOrchestrator(TournamentManager):
 
 
 def standings(tournament: models.TournamentInfo) -> list[tuple[int, models.PlayerInfo]]:
-    def sort_key(p):
+    def sort_key(p: models.Player):
         return (
-            p.state == models.PlayerState.FINISHED,
+            # dropouts go last (only matters when tournament in progress)
+            int(p.state == models.PlayerState.FINISHED),
+            # winner first
             -int(p.uid == tournament.winner),
+            # then finalists (higher score can have dropped out)
+            -int(p.uid in tournament.finals_seeds),
             -p.result.gw,
             -p.result.vp,
             -p.result.tp,
@@ -1051,7 +1209,7 @@ def ratings(tournament: models.TournamentInfo) -> dict[str, models.TournamentRat
         elif rank <= 10:
             gp_points = (10 - rank) + 6
         ret[player.uid] = models.TournamentRating(
-            tournament=models.TournamentMinimal(**dataclasses.asdict(tournament)),
+            tournament=models.TournamentRef(**dataclasses.asdict(tournament)),
             size=size,
             rounds_played=player.rounds_played,
             result=player.result,
@@ -1137,3 +1295,48 @@ def optimise_table_seating(
     for i, uid in enumerate(rounds[-1][table]):
         tournament.rounds[-1].tables[table].seating[i].player_uid = uid
     return score
+
+def deck_infos(tournament: models.Tournament, member: models.Person) -> list[models.DeckInfo]:
+    res = []
+    if tournament.multideck:
+        for idx, round_ in enumerate(tournament.rounds, 1):
+            finals = idx == len(tournament.rounds)
+            for table in round_.tables:
+                for seat in table.seating:
+                    if seat.deck:
+                        res.append(
+                            models.DeckInfo(
+                                deck=seat.deck,
+                                score=seat.result,
+                                winner=(
+                                    finals and seat.player_uid == tournament.winner
+                                ),
+                                finalist=(
+                                    finals
+                                    and seat.player_uid in tournament.finals_seeds
+                                ),
+                            )
+                        )
+    else:
+        for player in tournament.players.values():
+            if player.deck:
+                res.append(
+                    models.DeckInfo(
+                        deck=player.deck,
+                        score=player.result,
+                        winner=(player.uid == tournament.winner),
+                        finalist=(player.uid in tournament.finals_seeds),
+                    )
+                )
+    res.sort(
+        key=lambda info: (-int(info.winner), -int(info.finalist), info.score)
+    )
+    if can_admin_tournament(member, tournament):
+        return res
+    if tournament.decklists_mode == models.DeckListsMode.ALL:
+        return res
+    if tournament.decklists_mode == models.DeckListsMode.FINALISTS:
+        res.decks = [d for d in res.decks[:5] if d.finalist]
+        return res
+    res.decks = [d for d in res.decks[:1] if d.winner]
+    return res
