@@ -11,6 +11,7 @@ from . import scoring
 
 class TournamentFormat(enum.StrEnum):
     Standard = "Standard"
+    V5 = "V5"
     Limited = "Limited"
     Draft = "Draft"
 
@@ -23,6 +24,7 @@ class TournamentRank(enum.StrEnum):
 
 
 class TournamentState(enum.StrEnum):
+    PLANNED = "Planned"
     REGISTRATION = "Registration"
     WAITING = "Waiting"
     PLAYING = "Playing"
@@ -37,6 +39,12 @@ class StandingsMode(enum.StrEnum):
     PUBLIC = "Public"  # All players
 
 
+class DeckListsMode(enum.StrEnum):
+    WINNER = "Winner"  # Default
+    FINALISTS = "Finalists"
+    ALL = "All"
+
+
 class PlayerState(enum.StrEnum):
     REGISTERED = "Registered"
     CHECKED_IN = "Checked-in"
@@ -70,6 +78,11 @@ class LeagueRanking(enum.StrEnum):
     Score = "Score"
 
 
+class LeagueKind(enum.StrEnum):
+    LEAGUE = "League"
+    META = "Meta-League"
+
+
 class MemberRole(enum.StrEnum):
     ADMIN = "Admin"
     PRINCE = "Prince"
@@ -90,13 +103,14 @@ class PublicPerson:
     country: str | None = ""  # country name
     country_flag: str | None = ""  # unicode flag char
     city: str | None = ""  # city name
+    roles: list[MemberRole] = pydantic.Field(default_factory=list)
 
 
 @dataclasses.dataclass(kw_only=True)
 class TournamentRef:
     name: str
     uid: str = pydantic.Field(default_factory=lambda: str(uuid.uuid4()))
-    format: TournamentFormat
+    format: TournamentFormat = TournamentFormat.Standard
     online: bool = False
     start: datetime.datetime
     timezone: str = "UTC"
@@ -120,7 +134,6 @@ class RegisteredSanction(Sanction):
 @dataclasses.dataclass
 class Person(PublicPerson):
     nickname: str | None = None  # player nickname (on social, lackey, etc.)
-    roles: list[MemberRole] = pydantic.Field(default_factory=list)
     sponsor: str | None = ""  # useful for organizers, to find their recruits
     sanctions: list[RegisteredSanction] = pydantic.Field(default_factory=list)
     ranking: dict[RankingCategoy, int] = pydantic.Field(default_factory=dict)
@@ -155,6 +168,12 @@ class Person(PublicPerson):
         return list(roles)
 
 
+@dataclasses.dataclass
+class PersonsUpdate:
+    update: list[Person] = pydantic.Field(default_factory=list)
+    delete: list[str] = pydantic.Field(default_factory=list)
+
+
 @dataclasses.dataclass
 class KrcgCard:
     id: int
@@ -188,16 +207,8 @@ class KrcgDeck:
     library: KrcgLibrary
     id: str = pydantic.Field(default_factory=lambda: str(uuid.uuid4()))
     vdb_link: str = ""
-    event: str = ""
-    event_link: str = ""
-    place: str = ""
-    date: datetime.date | None = None
-    tournament_format: str = ""
-    players_count: int = 0
-    player: str = ""
-    score: str = ""
     name: str = ""
-    author: str = ""
+    author: str = ""  # for attribution
     comments: str = ""
 
 
@@ -210,6 +221,7 @@ class PlayerInfo(PublicPerson):
     result: scoring.Score = pydantic.Field(default_factory=scoring.Score)
     seed: int = 0  # Finals seed
     toss: int = 0  # non-zero when draws for seeding finals
+    rating_points: int | None = None
 
 
 @dataclasses.dataclass
@@ -230,6 +242,7 @@ class Player(Person):
     barriers: list[Barrier] = pydantic.Field(default_factory=list)
     toss: int = 0  # non-zero when draws for seeding finals
     deck: KrcgDeck | None = None  # card ID: card count, default deck (monodeck)
+    rating_points: int | None = None
 
     def __hash__(self):
         return hash(self.vekn)
@@ -291,17 +304,23 @@ class LeagueRef:
 
 
 @dataclasses.dataclass(kw_only=True)
-class League(LeagueRef):
+class LeagueMinimal(LeagueRef):
     start: datetime.datetime
     timezone: str
     format: TournamentFormat
     ranking: LeagueRanking
+    kind: LeagueKind = LeagueKind.LEAGUE
     finish: datetime.datetime | None = None
-    description: str = ""
     online: bool = False
     country: str | None = None
     country_flag: str | None = None
     organizers: list[PublicPerson] = pydantic.Field(default_factory=list)
+    parent: LeagueRef | None = None
+
+
+@dataclasses.dataclass(kw_only=True)
+class League(LeagueMinimal):
+    description: str = ""
 
 
 @dataclasses.dataclass(kw_only=True)
@@ -310,7 +329,7 @@ class TournamentMinimal(TournamentRef):
     country: str | None = None
     country_flag: str | None = None
     league: LeagueRef | None = None
-    state: TournamentState = TournamentState.REGISTRATION
+    state: TournamentState = TournamentState.PLANNED
 
 
 @dataclasses.dataclass(kw_only=True)
@@ -325,6 +344,7 @@ class TournamentConfig(TournamentMinimal):
     decklist_required: bool = True
     description: str = ""
     standings_mode: StandingsMode = StandingsMode.PRIVATE
+    decklists_mode: DeckListsMode = DeckListsMode.WINNER
     max_rounds: int = 0
     limited: LimitedFormat | None = None
 
@@ -352,7 +372,7 @@ class Tournament(TournamentConfig):
 
 @dataclasses.dataclass
 class TournamentInfo(TournamentConfig):
-    players: dict[str, PlayerInfo] = pydantic.Field(default_factory=dict)
+    players: dict[str, Player | PlayerInfo] = pydantic.Field(default_factory=dict)
     finals_seeds: list[str] = pydantic.Field(default_factory=list)
     rounds: list[RoundInfo] = pydantic.Field(default_factory=list)
     winner: str = ""
@@ -365,6 +385,9 @@ class TournamentFilter(pydantic.BaseModel):
     country: str = ""
     online: bool = True
     states: list[TournamentState] = pydantic.Field(default_factory=list)
+    member_uid: str = ""
+    year: int | None = None
+    name: str | None = None
 
 
 @dataclasses.dataclass
@@ -377,6 +400,7 @@ class VenueCompletion:
 
 @dataclasses.dataclass
 class LeagueWithTournaments(League):
+    leagues: list[LeagueMinimal] = pydantic.Field(default_factory=list)
     tournaments: list[TournamentInfo] = pydantic.Field(default_factory=list)
     rankings: list[tuple[int, LeaguePlayer]] = pydantic.Field(default_factory=list)
 
@@ -394,6 +418,7 @@ class DeckInfo:
     deck: KrcgDeck
     score: scoring.Score
     winner: bool = False
+    finalist: bool = False
 
 
 @dataclasses.dataclass
@@ -457,6 +482,7 @@ class Member(PersonWithRatings):
     password_hash: str = ""
     whatsapp: str | None = None  # phone
     prefix: str | None = None  # temporary, to compute sponsors when syncing vekn
+    authorized_clients: dict[str, dict] = pydantic.Field(default_factory=dict)  # client_uid -> {authorized_at: timestamp}
 
 
 @dataclasses.dataclass
